-- name: [CS] Generic Soldier
-- description: Ready for deployment!
--[[
    API Documentation for Character Select can be found below:
    https://github.com/Squishy6094/character-select-coop/blob/main/API-doc.md
]]

local TEXT_MOD_NAME = ""
local TEXT_MOD_VERSION = "1.0"
local globalModName = "Generic_Soldier"

local TEX_SOLDIERICON = get_texture_info("Soldiericon")

local SOLDIER_SOUNDBANK = {
	[CHAR_SOUND_LETS_A_GO] = {'soldier_letsago2.ogg', "soldier_letsago.ogg"},
	[CHAR_SOUND_PUNCH_YAH] = 'soldier_punch1.ogg',
	[CHAR_SOUND_PUNCH_WAH] = 'soldier_punch2.ogg',
	[CHAR_SOUND_PUNCH_HOO] = 'soldier_punch3.ogg',
	[CHAR_SOUND_YAH_WAH_HOO] = {'soldier_yah2.ogg', 'soldier_yah.ogg', 'soldier_yah3.ogg'},
	[CHAR_SOUND_HOOHOO] = {'soldier_hoohoo2.ogg', 'soldier_hoohoo.ogg'},
	[CHAR_SOUND_YAHOO_WAHA_YIPPEE] = {'soldier_yippee.ogg', 'soldier_yippee2.ogg'},
	[CHAR_SOUND_UH] = {'soldier_uh.ogg', 'soldier_uh2.ogg', 'soldier_uh3.ogg'},
	[CHAR_SOUND_HAHA] = {'soldier_haha.ogg', 'soldier_haha2.ogg', 'soldier_haha3.ogg', 'soldier_haha4.ogg', 'soldier_haha5.ogg'},
	[CHAR_SOUND_YAHOO] = {'soldier_yahoo.ogg', 'soldier_yahoo2.ogg'},
	[CHAR_SOUND_DOH] = {'soldier_doh.ogg', 'soldier_doh2.ogg', 'soldier_doh3.ogg'},
	[CHAR_SOUND_WHOA] = {'soldier_woah.ogg', 'soldier_woah2.ogg', 'soldier_woah3.ogg', 'soldier_woah4.ogg'},
	[CHAR_SOUND_EEUH] = {'soldier_eeuh.ogg', 'soldier_hrrm.ogg'},
	[CHAR_SOUND_WAAAOOOW] = {'soldier_falling.ogg', 'soldier_falling2.ogg', 'soldier_falling3.ogg'},
	[CHAR_SOUND_HRMM] = 'soldier_hrrm.ogg',
	[CHAR_SOUND_HERE_WE_GO] = {'soldier_herewego2.ogg', 'soldier_herewego.ogg', 'soldier_herewego3.ogg', 'soldier_herewego3.ogg'},
	[CHAR_SOUND_SO_LONGA_BOWSER] = 'soldier_solongbowser.ogg',
	[CHAR_SOUND_ATTACKED] = 'soldier_attacked.ogg',
	[CHAR_SOUND_PANTING] = 'soldier_panting.ogg',
	[CHAR_SOUND_ON_FIRE] = {'soldier_onfire2.ogg', 'soldier_onfire.ogg'},
	[CHAR_SOUND_DYING] = {'soldier_dying.ogg', 'soldier_dying2.ogg'},
	[CHAR_SOUND_MAMA_MIA] = {'soldier_mamamia.ogg', 'soldier_mamamia2.ogg'},
}

local E_MODEL_ODST = smlua_model_util_get_id("ODST_geo")
local E_MODEL_HELLDIVER = smlua_model_util_get_id("Helldiver_geo")
local E_MODEL_COG = smlua_model_util_get_id("COG_geo")

local CT_RECRUIT = _G.charSelect.character_add("Generic Soldier","reporting for duty, Sir!", "Ash_Must_Die", { r = 255, g = 255, b = 255 }, E_MODEL_ODST, CT_RECRUIT, TEX_SOLDIERICON)

_G.charSelect.character_add_voice(E_MODEL_ODST, SOLDIER_SOUNDBANK)
_G.charSelect.character_add_voice(E_MODEL_HELLDIVER, SOLDIER_SOUNDBANK)
_G.charSelect.character_add_voice(E_MODEL_COG, SOLDIER_SOUNDBANK)

local character_get_voice, update_sound, update_snore = _G.charSelect.character_get_voice, _G.charSelect.voice.sound, _G.charSelect.voice.snore
local function character_sounds(m, sound)
    local v = _G.charSelect.character_get_voice(m)
    if v == SOLDIER_SOUNDBANK
    then
        return update_sound(m, sound)
    end
end
local function character_snore(m)
    local v = _G.charSelect.character_get_voice(m)
    if v == SOLDIER_SOUNDBANK
    then
        return update_snore(m)
    end
end

hook_event(HOOK_CHARACTER_SOUND, character_sounds)
hook_event(HOOK_MARIO_UPDATE, character_snore)

ANGLE_QUEUE_SIZE = 9
SPIN_TIMER_SUCCESSFUL_INPUT = 4

gEventTable = {}

gStateExtras = {}
for i = 0, (MAX_PLAYERS - 1) do
    gStateExtras[i] = {}
    local m = gMarioStates[i]
    local e = gStateExtras[i]
    e.prevPos = {}
    e.prevPos.x = 0
    e.prevPos.y = 0
    e.prevPos.z = 0
    e.angleDeltaQueue = {}
    for j = 0, (ANGLE_QUEUE_SIZE - 1) do e.angleDeltaQueue[j] = 0 end
    e.lastAction = m.action
    e.animFrame = 0
    e.animArg = 0
    e.scuttle = 0
    e.averageForwardVel = 0
    e.boostTimer = 0
    e.rotAngle = 0
    e.lastHurtCounter = 0
    e.stickLastAngle = 0
    e.spinDirection = 0
    e.spinBufferTimer = 0
    e.spinInput = 0
    e.lastIntendedMag = 0
    e.swims = 0
end

local charmov = 1

local character_edit = _G.charSelect.character_edit
local character_get_current_number = _G.charSelect.character_get_current_number
local character_get_current_table = _G.charSelect.character_get_current_table
local get_options_status = _G.charSelect.get_options_status
local get_menu_color = _G.charSelect.get_menu_color
local hook_render_in_menu = _G.charSelect.hook_render_in_menu

local table_insert = table.insert
local play_sound = play_sound
local djui_hud_get_screen_width = djui_hud_get_screen_width
local djui_hud_measure_text = djui_hud_measure_text
local djui_hud_set_color = djui_hud_set_color
local djui_hud_set_resolution = djui_hud_set_resolution
local djui_hud_set_rotation = djui_hud_set_rotation
local djui_hud_set_font = djui_hud_set_font
local djui_hud_print_text = djui_hud_print_text
local djui_hud_render_rect = djui_hud_render_rect
local maxf = maxf
local math_min = math.min
local math_max = math.max
local math_sin = math.sin

local altCostumes = {
    [CT_RECRUIT] = {
        currSkin = 1,
        {name = "ODST", model = E_MODEL_ODST,},
		{name = "Helldiver", model = E_MODEL_HELLDIVER,},
		{name = "COG", model = E_MODEL_COG,},
	},

}

local function update_character_skin(currChar, currAlt)
    local currSkin = altCostumes[currChar][currAlt]
    if altCostumes[currChar].desc == nil then
        altCostumes[currChar].desc = character_get_current_table().description
    end
    local description = {}
    for i = 1, #altCostumes[currChar].desc do
        table_insert(description, altCostumes[currChar].desc[i])
    end
    if currSkin.name ~= "" then
        table_insert(description, "")
        table_insert(description, "Current Outfit: "..currSkin.name)
    end
    character_edit(currChar, nil, description, nil, nil, currSkin.model, nil, nil)
    local cameraToObject = gMarioStates[0].marioObj.header.gfx.cameraToObject
    play_sound(SOUND_MENU_MESSAGE_NEXT_PAGE, cameraToObject)
end

local inputLockTimer = 0
local buttonAnimTimer = 0

local MATH_DIVIDE_320 = 1/320

local latencyValueTable = {12, 6, 3}

local function hud_render()
    local width = djui_hud_get_screen_width() + 1.4
    local widthScale = maxf(width, 321.4) * MATH_DIVIDE_320
    local currChar = character_get_current_number()
    local charColors = character_get_current_table().color
    
    -- Mimick button swaying, which is missing from the paper code
    local buttonAnim = 0
    local charSelectAnim = get_options_status(_G.charSelect.optionTableRef.anims)
    if charSelectAnim > 0 then
        buttonAnimTimer = buttonAnimTimer + 1
        buttonAnim = math_sin(buttonAnimTimer * 0.05) * 2.5 + 5
    else
        buttonAnim = 10
    end

    local inputLockTimerTo = latencyValueTable[get_options_status(_G.charSelect.optionTableRef.inputLatency) + 1]

    if altCostumes[currChar] ~= nil then
        -- Render Mod Variond under CS version
        local menuColor = get_menu_color()
        djui_hud_set_color(menuColor.r, menuColor.g, menuColor.b, 255)
        djui_hud_set_font(FONT_TINY)
        local string = TEXT_MOD_NAME.." ("..TEXT_MOD_VERSION..")"
        djui_hud_print_text(string, width - 5 - djui_hud_measure_text(string)*0.5, 3, 0.5)

        ---@type Controller
        local c = _G.charSelect.controller
        local currAlts = altCostumes[currChar]

        djui_hud_set_font(FONT_NORMAL)
        djui_hud_set_resolution(RESOLUTION_N64)
        local inputLockTimerAnim = 0

        local buttonX = 20 * widthScale + buttonAnim
        local x1 = buttonX - 4
        local x2 = x1 + 73
        local y = 97

        if charSelectAnim > 0 then
            inputLockTimerAnim = inputLockTimer/inputLockTimerTo * 3
            x1 = x1 + math_min(inputLockTimerAnim, 0)
            x2 = x2 + math_max(inputLockTimerAnim, 0)
        end
        -- Left Arrow
        if currAlts.currSkin > 1 then
            djui_hud_set_color(charColors.r, charColors.g, charColors.b, 255)
            djui_hud_set_rotation(0x2000, 0.5, 0.5)
            djui_hud_render_rect(x1, y, 5, 5)
            djui_hud_set_color(0, 0, 0, 255)
            djui_hud_set_rotation(0x0000, 0.5, 0.5)
            djui_hud_render_rect(x1 + 2.5, y - 2, 6, 8)
            djui_hud_set_color(charColors.r, charColors.g, charColors.b, 255)
            djui_hud_render_rect(buttonX, y - 3, 1, 10)

            if inputLockTimer == 0 and (c.buttonDown & L_JPAD ~= 0 or c.stickX < -0.5) then
                currAlts.currSkin = math_max(currAlts.currSkin - 1, 1)
                update_character_skin(currChar, currAlts.currSkin)
                inputLockTimer = -inputLockTimerTo
            end
        end
        -- Right Arrow
        if currAlts.currSkin < #currAlts then
            -- Changed arrows to mimick the ones seen in the options menu
            djui_hud_set_color(charColors.r, charColors.g, charColors.b, 255)
            djui_hud_set_rotation(0x2000, 0.5, 0.5)
            djui_hud_render_rect(x2, y, 5, 5)
            djui_hud_set_color(0, 0, 0, 255)
            djui_hud_set_rotation(0x0000, 0.5, 0.5)
            djui_hud_render_rect(x2 - 3.5, y - 2, 6, 8)
            djui_hud_set_color(charColors.r, charColors.g, charColors.b, 255)
            djui_hud_render_rect(buttonX + 69, y - 3, 1, 10)

            if inputLockTimer == 0 and (c.buttonDown & R_JPAD ~= 0 or c.stickX > 0.5) then
                currAlts.currSkin = currAlts.currSkin + 1
                update_character_skin(currChar, currAlts.currSkin)
                inputLockTimer = inputLockTimerTo
            end
        end
        -- use an input lock to prevent cycling too fast
        if inputLockTimer > 0 then
            inputLockTimer = inputLockTimer - 1
        elseif inputLockTimer < 0 then
            inputLockTimer = inputLockTimer + 1
        end
    end
end

hook_render_in_menu(hud_render)
